C ***************************************************************************
C
C   PROGRAM SEEINGCOR
C
C  This program computes the seeing corrected values of Dn, Re and <SBe> 
C  given the (seeing-affected) values Dnc, Rec, <SBe> and the full-width
C  half-maximum value of the seeing FWHM. It is assumed that the true galaxy 
C  profile is an R^1/4 law and that the stellar PSF follows a gamma-model,
C  according to Saglia et al. 1993, MNRAS, 264, 961
C
C  Roberto Philip Saglia
C  Universitaetssternwarte Muenchen
C  Scheinerstrasse 1
C  D-81673 Muenchen
C  email: saglia@hal1.usm.uni-muenchen.de
C ***************************************************************************
c
	program seeingcor
c
	implicit double precision (a-h,o-z)
	parameter (pi=3.1415926535898d0)
	parameter (nap=31,npsf=100)
	dimension dsbl(nap,npsf),ddsbl(nap,npsf)
	dimension dmag(nap,npsf),ddmag(nap,npsf)
	dimension dm(nap),dmlu(nap),ddm(nap),ddmlu(nap)
	dimension ddum1(nap),ddum2(nap)
	dimension y2are(npsf),y2adn(npsf)
	dimension reo(2000),dno(2000),fwhm(2000)
	dimension retrue(npsf),reobs(npsf)
	dimension dntrue(npsf),dnsee(npsf)
	dimension dntrue_tmp(npsf),dnsee_tmp(npsf)
	common /psicom1/ apmin,dlnap,dfwhm,dsbl,ddsbl,dmag,ddmag
	common /dev/ apmin0,dlnap0
	common /warn/ iwarn1,iwarn2
	character*15 tabcor
	character*80 filenam
	character*10 filena
	character*80 str
	external flux0
c
	call startup
c
c  Read set of data
c
	write(*,*)'Give data to be corrected'
	write(*,*)'(1): line input; (2) file input; [1]'
	iline=1
	read(*,*)iline
	if(iline.eq.1)then
	   write(*,*)'Give observed values Reo, Dno, FWHM in arcsec'
           write(*,*)' [10, 10, 2]'
	   nobj=1
	   reo(1)=10.
	   dno(1)=10.
	   fwhm(1)=2.
	   read(*,*)reo(1),dno(1),fwhm(1)
	   write(*,*)'Output values in recdncfwhm.seeingcor'
	   open(unit=98,file='recdncfwhm.seeingcor',status='unknown')
	else
	   write(*,*)'Give name of file containing Reo, Dno, FWHM'
	   write(*,*)'values in arcsec [recdncfwhm.dat]' 
	   write(*,*)'(Lines beginning with # are comments)'
	   filenam='recdncfwhm.dat'
	   read(*,'(a)')filenam
	   do i=1,80
	      if(filenam(i:i).eq.' ')then
		 ispace=i-1
		 goto 222
	      endif
	   enddo
 222	   write(*,'(a)')'Output values in '//filenam(1:ispace)//
     1                   '.seeingcor'
	   open(unit=99,file=filenam,status='old')
	   open(unit=98,file=filenam(1:ispace)//'.seeingcor',
     1          status='unknown')
	   iobj=0
	   do 99 i=1,2000
	      read(99,'(a)',end=98)str
	      if(str.eq.'                                        '//
     1                  '                                        ')
     2        goto 98
	      if(str(1:1).eq.'#')goto 99
	      iobj=iobj+1
	      if(iobj.gt.2000)then
		 write(*,*)'There are more than 2000 objects'
		 write(*,*)'Stopped at 2000'
		 goto 98
	      endif
	      read(str,*)reo(iobj),dno(iobj),fwhm(iobj)
	      write(*,*)reo(iobj),dno(iobj),fwhm(iobj)
99	continue
98	nobj=iobj-1
c
	write(*,*)'Number of objects to be corrected:',nobj
	close(unit=99)
	endif
c
c  Read table of corrections for Re
c
	itabcor=9
	write(*,*)'Give table for the seeing corrections [9]'
	write(*,*)'(1) gamma=1.3, (2) gamma=1.4, (3) gamma=1.5'
	write(*,*)'(4) gamma=1.6, (5) gamma=1.7, (6) gamma=1.8'
	write(*,*)'(7) gamma=1.9, (8) gamma=2.0, (9) gamma=5/3'
	read(*,*)itabcor
	if(itabcor.eq.1)tabcor='reseedev_1.3'
	if(itabcor.eq.2)tabcor='reseedev_1.4'
	if(itabcor.eq.3)tabcor='reseedev_1.5'
	if(itabcor.eq.4)tabcor='reseedev_1.6'
	if(itabcor.eq.5)tabcor='reseedev_1.7'
	if(itabcor.eq.6)tabcor='reseedev_1.8'
	if(itabcor.eq.7)tabcor='reseedev_1.9'
	if(itabcor.eq.8)tabcor='reseedev_2.0'
	if(itabcor.eq.9)tabcor='reseedev_5.3'
	open(unit=99,file=tabcor,status='old')
	    do 330 ire=1,npsf
	     read(99,*,end=97)retrue(npsf+1-ire),reobs(npsf+1-ire)
c	     write(*,*)retrue(npsf+1-ire),reobs(npsf+1-ire)
330	    continue
97	    nre=ire-1
	close(unit=99)
c
	call sbini(itabcor,filena)
	apmin0=apmin
	dlnap0=dlnap
	ratmean=0.d0
c
	write(98,*)'# Seeing Tables '//tabcor//' + '//filena   
        write(98,*)'# Reo      Dno       FWHM     Re      Dn       '//
     1             'd<SBe>'
        write(98,*)'# (")      (")       (")      (")     (")      '//
     1             '(mag/arcsec^2)'
c
	der2=(retrue(2)-retrue(1))/(reobs(2)-reobs(1))
	der1=(retrue(nre)-retrue(nre-1))/(reobs(nre)-reobs(nre-1))
	call spline(reobs,retrue,npsf,der1,der2,y2are)
c  Loop on objects
	  do 440 iobj=1,nobj
c  Compute corrected Re
	    if(reo(iobj).gt.reobs(npsf))then
	     recorr=reo(iobj)
	    else
	     if(reo(iobj)/fwhm(iobj).le.reobs(1)/1.5)then
	      recorr=reo(iobj)*retrue(1)/reobs(1)
	      write(*,*)'Reo/FWHM<=1, using Reo=FWHM'
	     else
	      recorr=fwhm(iobj)/1.5*
     1           splint(reobs,retrue,y2are,npsf,reo(iobj)/
     2                  fwhm(iobj)*1.5)
	     endif
	   endif
c	   write(*,*)'Re obs',reo(iobj),'Re corr',recorr 
c  Compute corrected Dn, iterating on Dn/Re
	if(dno(iobj).gt.150.d0)then
	 dncorr=dno(iobj)
	 goto 50
	endif
	dncorrold=dno(iobj)
c	write(*,*)'First Dn',dncorrold
	iter=0
60	rre0=dncorrold/recorr/2.d0
	sbm0=flux0(rre0)/(pi*rre0*rre0)
c	write(*,*) 'Dn(0)/Ae, sbm0=',real(rre0),real(sbm0)
c  Look over Re (galaxies of different sizes).
	  do 40 ipsf=1,npsf
	   fwhmre=2.0*ipsf*dfwhm
	   re=fwhm(iobj)/fwhmre
	    do 30 iap=1,nap
	     dm(iap)=dmag(iap,ipsf)
	     ddm(iap)=ddmag(iap,ipsf)
	     ap=apmin*exp((iap-1)*dlnap)
	     r=0.5*ap*fwhm(iobj)
30	    continue
	   call splder(ddum1,ddum2,nap)
	   dntrue_tmp(ipsf)=2.0*re*rre0
c  Now find r such that sbmsee=sbm0.
	   call findr(sbm0,fwhm(iobj),re,dm,ddm,nap,r)
	   if(r.eq.0)then
	    if(dntrue_tmp(ipsf).gt.50.d0)then
	     dnsee_tmp(ipsf)=dntrue_tmp(ipsf)
	    else
	     goto 110
	    endif
	   else
	    dnsee_tmp(ipsf)=2.0*r
	   endif
c	   write(*,*)dntrue_tmp(ipsf),dnsee_tmp(ipsf)
40	continue
110	nsee=ipsf-1
	do 111 isee=1,nsee
	 dntrue(isee)=dntrue_tmp(nsee+1-isee)
	 dnsee(isee)=dnsee_tmp(nsee+1-isee)
c	 write(*,*)dntrue(isee),dnsee(isee)
111	continue
c
c  Find true Dn
	if(reo(iobj)/fwhm(iobj).le.reobs(1)/1.5)then
	 dncorr=dno(iobj)*dntrue(1)/dnsee(1)
	else
	 der2dn=(dntrue(2)-dntrue(1))/(dnsee(2)-dnsee(1))
	 der1dn=(dntrue(nsee)-dntrue(nsee-1))/(dnsee(nsee)-dnsee(nsee-1))
	 call spline(dnsee,dntrue,nsee,der1dn,der2dn,y2adn)
         dncorr=splint(dnsee,dntrue,y2adn,nsee,dno(iobj))
	endif
	 err=abs((dncorr-dncorrold)/dncorr)
	 if(err.lt.0.005d0)then
c	  write(*,*)'Iteration',iter+1,' Error',err
	  goto 50
	 else
	  dncorrold=dncorr
	  iter=iter+1
c	  write(*,*)'Iteration ',iter,' Error ',err
	  if(iter.gt.100)then
	     write(*,*)'More than 100 iterations, stopped'
	     write(*,*)'Estimated relative error ',err
	     goto 50
	  else
	     goto 60
	  endif
	 endif
50	 ratmean=ratmean+dno(iobj)/dncorr
c
c Seeing correction to be added to the mean effective surface brightness
c d<SBe>=-5log10(Reo/Re)
c
	 dsbem=-5.*log10(reo(iobj)/recorr)
        write(*,*)'# Reo      Dno       FWHM     Re      Dn       '//
     1             'd<SBe>'
        write(*,*)'# (")      (")       (")      (")     (")      '//
     1             '(mag/arcsec^2)'
	write(*,'(6(x,f8.3))')
     1             real(reo(iobj)),real(dno(iobj)),real(fwhm(iobj)),
     2		   real(recorr),real(dncorr),real(dsbem)
	write(98,'(6(x,f8.3))')
     1             real(reo(iobj)),real(dno(iobj)),real(fwhm(iobj)),
     2		   real(recorr),real(dncorr),real(dsbem)
440	continue
	close(unit=98)
	end

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
	subroutine startup
c
	write(*,*)'****************************************************'
	write(*,*)'*                                                  *'
	write(*,*)'*             PROGRAM SEEINGCOR                    *'
	write(*,*)'*                                                  *'
	write(*,*)'****************************************************'
	write(*,*)
	write(*,*)'This program computes the seeing corrected values'
	write(*,*)'of Dn, Re and <SBe> given the observed (and'
	write(*,*)'seeing-affected) values Dno, Reo, <SBe> and the'
	write(*,*)'full-width half-maximum value of the seeing FWHM.'
	write(*,*)'It is assumed that the true galaxy profile is'
	write(*,*)'an R^1/4 law and that the stellar PSF follows'
	write(*,*)' a gamma-model, according to Saglia et al. 1993,'
        write(*,*)' MNRAS, 264, 961'
	write(*,*)
	write(*,*)'Roberto Philip Saglia'
	write(*,*)'Universitaetssternwarte Muenchen'
	write(*,*)'Scheinerstrasse 1'
	write(*,*)'D-81673 Muenchen'
	write(*,*)'email: saglia@hal1.usm.uni-muenchen.de'
	write(*,*)
	write(*,*)'****************************************************'
	write(*,*)
	write(*,*)'To use default values given between [], type /'
	write(*,*)'and press Return'
	write(*,*)
c
	return
	end
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
	subroutine findr(sbm0,fwhm,re,dm,ddm,nap,r)
c  Find r such that the mean surface brightness interior to r equals sbm0.
c  dm(logr) (with spline derivative ddm(logr)) gives the correction to
c  aperture magnitudes for the r**1/4 law.
c
	implicit double precision (a-h,o-z)
	dimension dm(nap),ddm(nap)
	common /dev/ apmin,dlnap
c
c  Find the zero of f(r):== dm(r)-2.5*log10(re**2*flux(r/re)/pi/r**2/sbm0).
c  First find the interval containing the root.  df/dr > 0.
c  Instead of r, use z=log(apfwhm/apmin)/dlnap=log(2.0*r/fwhm/apmin)/dlnap.
	iz=1
	  do 10 iap=1,nap
	  z=iap-1
	  frr=fr(sbm0,fwhm,re,dm,ddm,nap,z)
	  if (frr.lt.0.0) iz=iap
10	continue
c  Now know the root lies in (iz,iz+1).  Use Secant method to converge.
	z0=iz-1
	z1=iz
	f0=fr(sbm0,fwhm,re,dm,ddm,nap,z0)
	f1=fr(sbm0,fwhm,re,dm,ddm,nap,z1)
	if (f0.gt.0.0.or.f1.lt.0.0) then
c	  write(*,*) 'Error in findr!'
c	  write(*,*) 'z0,z1,f0,f1=',z0,z1,f0,f1
	  r=0.0
	  return
	end if
	dfdz=f1-f0
	dz=-f0/dfdz
	z=z0+dz
	frr=fr(sbm0,fwhm,re,dm,ddm,nap,z)
	niter=0
20	  niter=niter+1
	  if (frr.gt.0.0) then
	    z1=z
	    f1=frr
	  else
	    z0=z
	    f0=frr
	  end if
	  dfdz=(f1-f0)/(z1-z0)
	  dz=-f0/dfdz
	  z=z0+dz
	  frr=fr(sbm0,fwhm,re,dm,ddm,nap,z)
	  if (abs(dz).lt.1.0e-3) go to 30
	  if (niter.lt.1000) go to 20
	  write(*,*) 'Error in findr!  niter=1000'
30	continue
	r=0.5*exp(z*dlnap)*apmin*fwhm
c
	return
	end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
	function fr(sbm0,fwhm,re,dm,ddm,nap,apln)
c  f(r):== dm(r)-2.5*log10(re**2*flux(r/re)/pi/r**2/sbm0).
	implicit double precision (a-h,o-z)
	parameter (pi=3.1415926535898d0)
	dimension dm(nap),ddm(nap)
	common /dev/ apmin,dlnap
	external flux0
c
	x=apln+1.0
	ix=x
	if (ix.lt.1) then
	  ix=1
	else if (ix.ge.nap) then
	  ix=nap-1
	end if
	dx=x-ix
c  Linear interpolation in x.
c	dmr=(1.0-dx)*dm(ix)+dx*dm(ix+1)
c  Cubic splines interpolation in x.
	dmr=dm(ix)+dx*(ddm(ix)+dx*(3.0*(dm(ix+1)-dm(ix))-2.0*ddm(ix)
     2     -ddm(ix+1)+dx*(ddm(ix)+ddm(ix+1)+2.0*(dm(ix)-dm(ix+1)))))
	r=0.5*exp(apln*dlnap)*apmin*fwhm
	rre=r/re
	flux1=pi*rre*rre*sbm0
	fr=dmr-2.5*log10(flux0(rre)/flux1)
c
	return
	end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
	function sb0(r)
c  Compute the surface brightness distribution for the r**(1/4) law.
c  r is in units of the half-light radius Re.  Total flux is normalized to 1.
c
	implicit double precision (a-h,o-z)
	parameter (pi8=8.0d0*3.141592654d0,r0=3459.4850148d0)
c
	x=sqrt(sqrt(r*r0))
	sb0=r0*r0*exp(-x)/(5040.0d0*pi8)
	return
	end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
	function flux0(r)
c  Computes the integrated flux distribution for the r**(1/4) law.
c  r is in units of the half-light radius Re.  Total flux is normalized to 1.
c
	implicit double precision (a-h,o-z)
	parameter (r0=3459.4850148d0)
c
	x=sqrt(sqrt(r*r0))
	if (x.le.0.3) then
	  flux0=((r*r0)**2)/40320.0d0*exp(-x)*(1.0d0+x/9.0d0*(1.0d0+
     2 x/10.0d0*(1.0d0+x/11.0d0*(1.0d0+x/12.0d0*(1.0d0+x/13.0d0)))))
	else
	  flux0=(1.0d0-exp(-x)*(1.0d0+x*(1.0d0+x/2.0d0*(1.0d0+x/3.0d0*
     2 (1.0d0+x/4.0d0*(1.0d0+x/5.0d0*(1.0d0+x/6.0d0*
     3 (1.0d0+x/7.0d0))))))))
	end if
	return
	end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
	function sb(apfwhm,fwhmae)
c  Compute the surface brightness distribution for the r**(1/4) law,
c  _with_ seeing.
c  apfwhm = 2r/FWHM, fwhmae = FWHM/Ae, where Ae=2Re.
c  sb is calculated from linear interpolation of the 2-D table dsbl
c  read in from a seeing table.
c
	implicit double precision (a-h,o-z)
	parameter (nap=31,npsf=100)
	dimension dsbl(nap,npsf),ddsbl(nap,npsf)
	dimension dmag(nap,npsf),ddmag(nap,npsf)
	common /psicom1/ apmin,dlnap,dfwhm,dsbl,ddsbl,dmag,ddmag
	common /warn/ iwarn1,iwarn2
c
c  Do cubic splines interpolation on log(apfwhm) and linear interpolation on
c  fwhmae.
	y=fwhmae/dfwhm
	iy=y
	dy=y-iy
	if (iy.eq.0) then
	  write(*,*) 'FWHM too small in sb!  fwhmae=',fwhmae
c	  write(*,*) 'Warning: continuing with minimum allowed FWHM!'
	  write(*,*) 'Warning: extrapolating linearly!'
	  iwarn1=4
	  iy=1
c	  dy=0.0d0
	  dy=y-iy
	else if (iy.ge.npsf) then
	  write(*,*) 'FWHM too large in sb!  fwhmae=',fwhmae
	  write(*,*) 'Warning: continuing with maximum allowed FWHM!'
	  iwarn1=3
	  iy=npsf-1
	  dy=1.0
	end if
c
	x=log(apfwhm/apmin)/dlnap+1.0
	ix=x
	dx=x-ix
	if (ix.lt.1) then
c	  write(*,*) 'Aperture too small in sb!  apfwhm=',apfwhm
c	  write(*,*) 'Warning: extrapolating log-log!'
	  ix=1
	  dx=x-ix
	else if (ix.ge.nap) then
c	  write(*,*) 'Aperture too large in sb!  apfwhm=',apfwhm
c	  write(*,*) 'Warning: continuing with maximum allowed aperture!'
	  ix=nap-1
	  dx=1.0
	end if
c
c  Linear interpolation in x, for both iy and iy+1.
c	dsbl1=(1.0-dx)*dsbl(ix,iy)+dx*dsbl(ix+1,iy)
c	dsbl2=(1.0-dx)*dsbl(ix,iy+1)+dx*dsbl(ix+1,iy+1)
c  Cubic splines interpolation in x, for both iy and iy+1.
	dsbl1=dsbl(ix,iy)+dx*(ddsbl(ix,iy)+dx*(3.0*(dsbl(ix+1,iy)
     2       -dsbl(ix,iy))-2.0*ddsbl(ix,iy)-ddsbl(ix+1,iy)
     3       +dx*(ddsbl(ix,iy)+ddsbl(ix+1,iy)+2.0*(dsbl(ix,iy)
     4       -dsbl(ix+1,iy)))))
	iy=iy+1
	dsbl2=dsbl(ix,iy)+dx*(ddsbl(ix,iy)+dx*(3.0*(dsbl(ix+1,iy)
     2       -dsbl(ix,iy))-2.0*ddsbl(ix,iy)-ddsbl(ix+1,iy)
     3       +dx*(ddsbl(ix,iy)+ddsbl(ix+1,iy)+2.0*(dsbl(ix,iy)
     4       -dsbl(ix+1,iy)))))
c  Linear interpolation in y.
	dsbl0=(1.0-dy)*dsbl1+dy*dsbl2
c  Apply seeing correction to raw surface brightness.
	rre=apfwhm*fwhmae
	sb=sb0(rre)*exp(dsbl0)
c
	return
	end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
	function flux(apfwhm,fwhmae)
c  Compute the flux for the r**(1/4) law, _with_ seeing.
c  apfwhm = 2r/FWHM, fwhmae = FWHM/Ae, where Ae=2Re.
c  sb is calculated from linear interpolation of the 2-D table dsbl
c  read in from a seeing table.
c
	implicit double precision (a-h,o-z)
	parameter (nap=31,npsf=100)
	dimension dsbl(nap,npsf),ddsbl(nap,npsf)
	dimension dmag(nap,npsf),ddmag(nap,npsf)
	common /psicom1/ apmin,dlnap,dfwhm,dsbl,ddsbl,dmag,ddmag
	common /warn/ iwarn1,iwarn2
c
c  Do cubic splines interpolation on log(apfwhm) and linear interpolation on
c  fwhmae.
	y=fwhmae/dfwhm
	iy=y
	dy=y-iy
	if (iy.eq.0) then
	  write(*,*) 'FWHM too small in flux!  fwhmae=',fwhmae
c	  write(*,*) 'Warning: continuing with minimum allowed FWHM!'
	  write(*,*) 'Warning: extrapolating linearly!'
	  iwarn1=4
	  iy=1
c	  dy=0.0d0
	  dy=y-iy
	else if (iy.ge.npsf) then
	  write(*,*) 'FWHM too large in flux!  fwhmae=',fwhmae
	  write(*,*) 'Warning: continuing with maximum allowed FWHM!'
	  iwarn1=3
	  iy=npsf-1
	  dy=1.0
	end if
c
	x=log(apfwhm/apmin)/dlnap+1.0
	ix=x
	dx=x-ix
	if (ix.lt.1) then
c	  write(*,*) 'Aperture too small in flux!  apfwhm=',apfwhm
c	  write(*,*) 'Warning: extrapolating log-log!'
	  ix=1
	  dx=x-ix
	else if (ix.ge.nap) then
c	  write(*,*) 'Aperture too large in flux!  apfwhm=',apfwhm
c	  write(*,*) 'Warning: continuing with maximum allowed aperture!'
	  ix=nap-1
	  dx=1.0
	end if
c
c  Linear interpolation in x, for both iy and iy+1.
c	dmag1=(1.0-dx)*dmag(ix,iy)+dx*dmag(ix+1,iy)
c	dmag2=(1.0-dx)*dmag(ix,iy+1)+dx*dmag(ix+1,iy+1)
c  Cubic splines interpolation in x, for both iy and iy+1.
	dmag1=dmag(ix,iy)+dx*(ddmag(ix,iy)+dx*(3.0*(dmag(ix+1,iy)
     2       -dmag(ix,iy))-2.0*ddmag(ix,iy)-ddmag(ix+1,iy)
     3       +dx*(ddmag(ix,iy)+ddmag(ix+1,iy)+2.0*(dmag(ix,iy)
     4       -dmag(ix+1,iy)))))
	iy=iy+1
	dmag2=dmag(ix,iy)+dx*(ddmag(ix,iy)+dx*(3.0*(dmag(ix+1,iy)
     2       -dmag(ix,iy))-2.0*ddmag(ix,iy)-ddmag(ix+1,iy)
     3       +dx*(ddmag(ix,iy)+ddmag(ix+1,iy)+2.0*(dmag(ix,iy)
     4       -dmag(ix+1,iy)))))
c  Linear interpolation in y.
	dmag0=(1.0-dy)*dmag1+dy*dmag2
c  Apply seeing correction to raw surface brightness.
	rre=apfwhm*fwhmae
	flux=flux0(rre)*10.0d0**(-0.4d0*dmag0)
c
	return
	end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
	subroutine sbini(itabcor,filena)
c  Initialize surface brightness for the r**(1/4) law.
c
	implicit double precision (a-h,o-z)
	parameter (nap=31,npsf=100)
	dimension dsbl(nap,npsf),ddsbl(nap,npsf)
	dimension dmag(nap,npsf),ddmag(nap,npsf)
	dimension dum1(nap),dum2(nap),ddum1(nap),ddum2(nap)
	character*10 filena
	common /psicom1/ apmin,dlnap,dfwhm,dsbl,ddsbl,dmag,ddmag
c
	call splini
c  Initialize model with seeing.
c
	if(itabcor.eq.1)filena='seedev_1.3'
	if(itabcor.eq.2)filena='seedev_1.4'
	if(itabcor.eq.3)filena='seedev_1.5'
	if(itabcor.eq.4)filena='seedev_1.6'
	if(itabcor.eq.5)filena='seedev_1.7'
	if(itabcor.eq.6)filena='seedev_1.8'
	if(itabcor.eq.7)filena='seedev_1.9'
	if(itabcor.eq.8)filena='seedev_2.0'
	if(itabcor.eq.9)filena='seedev_5.3'
c
c	write(*,*)itabcor,filena
c
	open(10,file=filena,status='old')
	rewind 10
	read(10,*)
	read(10,*) nap0,npsf0,dlnap,dfwhm,expon
c	write(*,*)
c     2 'nap,npsf,dlnap,dfwhm,expon=',nap0,npsf0,dlnap,dfwhm,expon
	if (nap.ne.nap0.or.npsf.ne.npsf0) then
	  write(*,*) 'Seedev filename has wrong size!'
	  write(*,*) 'Expected nap, npsf=',nap,npsf
	  write(*,*) 'Found nap, npsf=',nap0,npsf0
	  stop
	end if
	  do 30 ipsf=1,npsf
	    do 20 iap=1,nap
	    read(10,*) fwhm,ap,sbr,dm
	    if (ipsf.eq.1.and.iap.eq.1) apmin=ap
	    dsbl(iap,ipsf)=log(sbr)
	    dmag(iap,ipsf)=dm
20	  continue
	    do 23 iap=1,nap
	    dum1(iap)=dsbl(iap,ipsf)
	    dum2(iap)=dmag(iap,ipsf)
23	  continue
	  call splder(dum1,ddum1,nap)
	  call splder(dum2,ddum2,nap)
	    do 26 iap=1,nap
	    ddsbl(iap,ipsf)=ddum1(iap)
	    ddmag(iap,ipsf)=ddum2(iap)
26	  continue
30	continue
	close(10)
c
	return
	end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
	subroutine splder(y,dy,n)
c  Splder fits a cubic spline to y and returns the first derivatives at
c  the grid points in dy.  Dy is equivalent to a 4th-order Pade
c  difference formula for dy/di.
c
	implicit double precision (a-h,o-z)
	dimension f(3001),g(3001),y(n),dy(n)
	common /splinec/ g
c
	n1=n-1
	if (n1.gt.3000) write(*,*) 'Spline array overflow!!! n1=',
     1                  n1,'>3000'
c  Quartic fit to dy/di at boundaries, assuming d3y/di3=0.
	f(1)=(-10.0d0*y(1)+15.0d0*y(2)-6.0d0*y(3)+y(4))/6.0d0
	f(n)=(10.0d0*y(n)-15.0d0*y(n1)+6.0d0*y(n-2)-y(n-3))/6.0d0
c  Solve the tridiagonal system
c  dy(i-1)+4*dy(i)+dy(i+1)=3*(y(i+1)-y(i-1)), i=2,3,...,n1,
c  with dy(1)=f(1), dy(n)=f(n).
	  do 10 i=2,n1
	  f(i)=g(i)*(3.0d0*(y(i+1)-y(i-1))-f(i-1))
10	continue
	dy(n)=f(n)
	  do 20 i=n1,1,-1
	  dy(i)=f(i)-g(i)*dy(i+1)
20	continue
	return
	end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
	subroutine splini
c  Splini must be called before splder to initialize array g in common.
c
	implicit double precision (a-h,o-z)
	dimension g(3001)
	common /splinec/ g
	save /splinec/
c
	g(1)=0.0d0
	  do 10 i=2,3001
	  g(i)=1.0d0/(4.0d0-g(i-1))
10	continue
	return
	end
*******************************************************************************
c
c	cubic spline
c	from Press at al. Numerical Recipes, pg. 40
c
	subroutine spline(x,y,n,yp1,ypn,y2)
c	
c	Yi=f(Xi), YP1, YPN = first derivatives at X1, XN
c	returns the array of second derivatives Y2
c
	implicit real*8 (a-h,o-z)
	dimension x(n),y(n),y2(n),u(500)
c		assign lower boundary condition
	if (yp1.gt.1.d30) then
		y2(1)=0.
		u(1)=0.
	else
		y2(1)=-0.5
		u(1)=(3./(x(2)-x(1)))*((y(2)-y(1))/(x(2)-x(1))-yp1)
	endif
c		tridiagonal decomposition loop
	do 11 i=2,n-1
		sig=(x(i)-x(i-1))/(x(i+1)-x(i-1))
		p=sig*y2(i-1)+2.
		y2(i)=(sig-1.)/p
		u(i)=(6.*((y(i+1)-y(i))/(x(i+1)-x(i))-(y(i)-y(i-1))
     1                /(x(i)-x(i-1)))/(x(i+1)-x(i-1))-sig*u(i-1))/p
 11	continue
c		assign upper boundary
	if (ypn.gt.1.d30) then
		qn=0.
		un=0.
	else
		qn=0.5
		un=(3./(x(n)-x(n-1)))*(ypn-(y(n)-y(n-1))/(x(n)-x(n-1)))
	endif
	y2(n)=(un-qn*u(n-1))/(qn*y2(n-1)+1.)
c		backsubstitution loop
	do 12 k=n-1,1,-1
		y2(k)=y2(k)*y2(k+1)+u(k)
 12	continue
c
	return
	end
c
c	given xa and ya tabulating a function and y2a produced by SPLINE
c	returns the value of y at the position x
c
	real*8 function splint(xa,ya,y2a,n,x)
	implicit real*8 (a-h,o-z)
	dimension xa(n),ya(n),y2a(n)
	klo=1
	khi=n
c		bisection
 1	if (khi-klo.gt.1) then
	k=(khi+klo)/2.
	if (xa(k).gt.x) then
		khi=k
	else
		klo=k
	endif
	goto 1
	endif
c
	h=xa(khi)-xa(klo)
	if (h.eq.0.) pause 'SPLINT *** bad XA input'
	a=(xa(khi)-x)/h
	b=(x-xa(klo))/h
	splint=a*ya(klo)+b*ya(khi)+
     1          ((a**3-a)*y2a(klo)+(b**3-b)*y2a(khi))*(h**2)/6.
c
	return
	end

